( 3iMenu.fs -- Assad Ebrahim - module providing experimental menu capabilities )

\ AVR dependencies: 3iOLED.avr
\ Forth dependencies: 3iOLED.fs

\ Hardware: Yellow-Blue OLED display 128x64 pixels,  16-top pixels are YELLOW, remaining 48 pixels are BLUE
\ DESIGN: to read hardware buttons in real-time, need to debounce AND maintain state so only 1 press gets registered...

\ 5-button menu controls use Nano pins 2-6
2 constant k1
3 constant k2
4 constant k3
5 constant k4
6 constant k5

: menu-init
    k1 pullup \ buttons connect pin to ground and are active low
    k2 pullup
    k3 pullup
    k4 pullup
    k5 pullup
;

: reset-menu oled-init 2 oled-textsz ;

\ Design: one beat per bit, either beat (1) or rest (0)
\         4 beats per note
\         2 notes per 8-bit byte
variable beatcount
variable bytecount  \ actually this is byte or half-bar counts...
: beatcount++ beatcount @ 1 + beatcount ! ;
: bytecount++  bytecount @ 1 + bytecount ! ;
: beatcount@ beatcount @ ;
: bytecount@ bytecount @ ;
: beatcount! beatcount ! ;
: bytecount! bytecount ! ;
: clearbeats beatcount@ dup if ndrop   0 beatcount! else drop then ;
: resetbeats 0 beatcount! ;
: resetbytes 0 bytecount! clearbeats ;

variable _stop
: stop 1 _stop ! ;
: reset-stop 0 _stop ! ;
: stop? _stop @ ;

: addbyte >b8 bytecount++ ;
: playbytes ( bytes -- ) bytecount@ Nbeat-msb ;

: menu-read ( -- )
\   TODO: add space after 4 note/rests
   k5 pinread not if note. 1 beatcount++ then
   k1 pinread not if rest. 0 beatcount++ then
   k2 pinread not if clearbeats reset-menu then  \ clear beats in current byte, but previous bytes unchanged
   k4 pinread not if resetbytes reset-menu then   \ clear bytes and beats
   k3 pinread not if stop then \ exit and play
   beatcount@ 8 = if cr. addbyte playbytes resetbeats then
   \ TODO: scroll a line after each second 8 beats
;

: beats ( -- ) 
    reset-menu
    resetbytes reset-stop 
    begin menu-read stop? key? or until 
    bytecount@ dup if Nseq-msb else drop then 
;



 
\ NEXT: extend to 1 byte 
\  add bytes
\  interface with SFX generator


\ =======================================
: help-menu
  cr
  cr ." MENUS (help-menu)"
  cr 
  cr ." include 3iForth.fs, 3iSensor.fs, 3iBeats.fs, 3iOled.fs, 3iMenu.fs, include 3iBeats_cookbooks.fs"
  cr ." include 3iRecord.fs, 3iShift.fs, 3iPianoLights.fs, 3iSound-4x4.fs
  cr ." RealTerm   1 open-com   rx"
  cr
  cr ." 14 start-speaker  init-beats  oled-init  menu-init  4 init-percustones"
  cr
  cr ." 125 beatdur!"
  cr ." beats  rasra3"
  cr
;

help-menu


\  ===========================
\ V1 - 2020-11-29---Sun---09:07 - started developing menu capability for OLED screen on Piano PCB
\ V2 - 2021-05-27---Thu---08:04 - reworked OLED.fs interface design to streamline it
\ V3 - - \ developing menu capability for 5-button Beat Sequencer - 
